%% Least Squares.

% How can we determine the relationship between the admission exam and a
% course grades for any student in a university? We have the following
% dataset obtained from 22 students:
D = [...
%- Exam grade ------ Course grade -%
    50                  53;...
    35                  41;...
    35                  61;...
    40                  56;...
    55                  68;...
    65                  36;...
    35                  11;...
    60                  70;...
    90                  79;...
    35                  59;...
    90                  54;...
    80                  91;...
    60                  48;...
    60                  71;...
    60                  71;...
    40                  47;...
    55                  53;...
    50                  68;...
    65                  57;...
    50                  79];
%---------------------------------%

x = D(:,1);             % Independent variable.
y = D(:,2);             % Dependent variable.

figure;
plot( x, y, 'bo', 'MarkerFaceColor', 'g', 'MarkerSize', 7 );
xlabel( 'x' );
ylabel( 'y' );
title( 'Mathematical relationship between admission exam grade and a course grade' );
xlim( [min(x)-10 max(x)+10] );
ylim( [min(y)-10 max(y)+10] );
axis equal;
hold on;

%% Solving the least squares problem for a cubic polynomial.
n = max( size(D) );
A = [ x.^3 x.^2, x.^1, x.^0 ];

% Ac = y <---- where c is a vector of four coefficients: a, b, c, and d for a
% cubic polynomial with equation y = ax^3 + bx^2 + cx + d
c = pinv(A) * y;

%% Showing the best fit for the input dataset.

xCurve = linspace( 0, 120, 30 )';
ACurve = [xCurve.^3 xCurve.^2 xCurve.^1 xCurve.^0];
yCurve = ACurve * c;
plot( xCurve, yCurve, 'm-' );
text( 5, 90, sprintf('y = %.3fx^3 + %.3fx^2 + %.3fx + %.3f', c(1), c(2), c(3), c(4) ),...
    'Color', 'm', 'FontSize', 12 );

%% Computing the error.

yNew = A * c;           % Approximated y.
for I = 1:n
    plot( [x(I) x(I)], [yNew(I) y(I)], '--', 'Color', [0.6,0.6,0.6] );
end;
plot( x, yNew, 'b.', 'MarkerSize', 10 );

error = norm( yNew - y )^2;
text( 80, 10, sprintf( 'Error = %.3f', error ), 'Color', 'k', 'FontSize', 12 );
hold off;
